package main

import (
	"bytes"
	"fmt"
	"io"
	"mime/multipart"
	"net/http"
	"os"
	"os/exec"
	"path/filepath"
	"time"

	"github.com/stianeikeland/go-rpio/v4"
)

func main() {
	discordWebhookURL, ok := os.LookupEnv("DISCORD_WEBHOOK_URL")
	if !ok {
		fmt.Fprintln(
			os.Stderr,
			"'DISCORD_WEBHOOK_URL' env var is required",
		)
		os.Exit(1)
	}

	pin := rpio.Pin(18)

	if err := rpio.Open(); err != nil {
		fmt.Println(err)
		os.Exit(1)
	}

	defer rpio.Close()

	pin.Input()
	pin.PullUp()
	pin.Detect(rpio.FallEdge)

	fmt.Println("Sensing Enabled.")

	perched := false
	inputCh := make(chan struct{})
	fileCh := make(chan string, 10)
	errCh := make(chan error, 1)
	resultCh := make(chan string, 10)

	go captureImage(inputCh, fileCh, errCh)
	go sendImage(discordWebhookURL, fileCh, resultCh, errCh)
	go sendImage(discordWebhookURL, fileCh, resultCh, errCh)

	go func() {
		for {
			select {
			case err := <-errCh:
				fmt.Fprintln(
					os.Stderr,
					"Error received:",
					err,
				)
			case result := <-resultCh:
				fmt.Println(result)
			}
		}

	}()

	for range time.Tick(500 * time.Millisecond) {
		if pin.EdgeDetected() {
			if perched {
				perched = false
				continue
			}

			perched = true
			fmt.Println("Bird Perched! Taking snapshot")

			inputCh <- struct{}{}
		}
	}
}

func sendImage(discordWebhookURL string, fileCh <-chan string, resultCh chan<- string, errCh chan<- error) {
	client := &http.Client{
		Timeout: 10 * time.Second,
	}

	for capture := range fileCh {
		multipartReq, err := newMultiPartRequest(capture, discordWebhookURL)
		if err != nil {
			errCh <- fmt.Errorf(
				"Failed to create file multipart request: %s",
				err,
			)
			continue
		}

		if err := sendRequest(multipartReq, client); err != nil {
			errCh <- fmt.Errorf(
				"Capture failed to post to Discord channel: %s",
				err,
			)
			continue
		}

		os.Remove(capture)
		resultCh <- "Capture posted to Discord channel"
	}
}

func captureImage(inputCh <-chan struct{}, fileCh chan<- string, errCh chan<- error) {
	for range inputCh {
		output, err := os.CreateTemp("", "capture*.jpg")
		if err != nil {
			errCh <- err
		}
		output.Close()

		cmd := exec.Command(
			"rpicam-still",
			"--width", "1024",
			"--height", "768",
			"-o", output.Name(),
		)
		if err := cmd.Run(); err != nil {
			errCh <- fmt.Errorf("failed to capture image: %s", err)
			os.Remove(output.Name())
			continue
		}

		fileCh <- output.Name()
	}
}

func newMultiPartRequest(path, discordWebhook string) (*http.Request, error) {
	f, err := os.Open(path)
	if err != nil {
		return nil, err
	}
	defer f.Close()

	body := bytes.Buffer{}
	bodywriter := multipart.NewWriter(&body)

	part, err := bodywriter.CreateFormFile(
		"multipart/form-data",
		filepath.Base(f.Name()),
	)
	if err != nil {
		return nil, err
	}

	io.Copy(part, f)
	bodywriter.Close()

	request, err := http.NewRequest(http.MethodPost, discordWebhook, &body)
	if err != nil {
		return nil, err
	}
	request.Header.Add("Content-Type", bodywriter.FormDataContentType())

	return request, nil
}

func sendRequest(request *http.Request, client *http.Client) error {
	response, err := client.Do(request)
	if err != nil {
		return err
	}
	defer response.Body.Close()

	if response.StatusCode != http.StatusOK {
		return fmt.Errorf(
			"invalid response from discord channel: %s",
			response.Status,
		)
	}

	return nil
}
