package main

import (
	"bytes"
	"fmt"
	"io"
	"mime/multipart"
	"net/http"
	"os"
	"os/exec"
	"path/filepath"
	"time"

	"github.com/stianeikeland/go-rpio/v4"
)

func main() {
	discordWebhookURL, ok := os.LookupEnv("DISCORD_WEBHOOK_URL")
	if !ok {
		fmt.Fprintln(
			os.Stderr,
			"'DISCORD_WEBHOOK_URL' env var is required",
		)
		os.Exit(1)
	}

	pin := rpio.Pin(18)

	if err := rpio.Open(); err != nil {
		fmt.Println(err)
		os.Exit(1)
	}

	defer rpio.Close()

	pin.Input()
	pin.PullUp()
	pin.Detect(rpio.FallEdge)

	fmt.Println("Sensing Enabled.")

	perched := false

	for range time.Tick(500 * time.Millisecond) {
		if pin.EdgeDetected() {
			if perched {
				perched = false
				continue
			}

			perched = true
			fmt.Println("Bird Perched! Taking snapshot")

			go captureSendImage(discordWebhookURL)
		}
	}
}

func captureSendImage(discordWebhookURL string) {
	capture, err := captureImage()
	if err != nil {
		fmt.Fprintln(os.Stderr, "Error capturing picture:", err)
		return
	}

	multipartReq, err := newMultiPartRequest(capture, discordWebhookURL)
	if err != nil {
		fmt.Fprintln(
			os.Stderr,
			"Failed to create file multipart request:",
			err,
		)
		return
	}

	if err := sendRequest(multipartReq); err != nil {
		fmt.Fprintln(
			os.Stderr,
			"Capture failed to post to Discord channel:",
			err,
		)
		return
	}

	fmt.Println("Capture posted to Discord channel")
	os.Remove(capture)
}

func captureImage() (string, error) {
	output, err := os.CreateTemp("", "capture*.jpg")
	if err != nil {
		return "", err
	}
	output.Close()

	cmd := exec.Command(
		"rpicam-still",
		"--width", "1024",
		"--height", "768",
		"-o", output.Name(),
	)
	if err := cmd.Run(); err != nil {
		return "", fmt.Errorf("failed to capture image: %s", err)
	}

	return output.Name(), nil
}

func newMultiPartRequest(path, discordWebhook string) (*http.Request, error) {
	f, err := os.Open(path)
	if err != nil {
		return nil, err
	}
	defer f.Close()

	body := bytes.Buffer{}
	bodywriter := multipart.NewWriter(&body)

	part, err := bodywriter.CreateFormFile(
		"multipart/form-data",
		filepath.Base(f.Name()),
	)
	if err != nil {
		return nil, err
	}

	io.Copy(part, f)
	bodywriter.Close()

	request, err := http.NewRequest(http.MethodPost, discordWebhook, &body)
	if err != nil {
		return nil, err
	}
	request.Header.Add("Content-Type", bodywriter.FormDataContentType())

	return request, nil
}

func sendRequest(request *http.Request) error {
	client := http.Client{
		Timeout: 10 * time.Second,
	}

	response, err := client.Do(request)
	if err != nil {
		return err
	}
	defer response.Body.Close()

	if response.StatusCode != http.StatusOK {
		return fmt.Errorf(
			"invalid response from discord channel: %s",
			response.Status,
		)
	}

	return nil
}
