package main

import (
	"bytes"
	"fmt"
	"io"
	"io/ioutil"
	"mime/multipart"
	"net/http"
	"os"
	"os/exec"
	"path/filepath"
	"time"
)

func main() {
	discordWebhook := os.Getenv("BWATCH_DISCORD_URL")

	if len(discordWebhook) == 0 {
		fmt.Fprintln(os.Stderr, "Discord URL is required. Set env var 'BWATCH_DISCORD_URL'")
		os.Exit(1)
	}

	imagePath, err := captureImage()
	if err != nil {
		fmt.Fprintln(os.Stderr, "Failed to capture image:", err)
		os.Exit(1)
	}
	defer os.Remove(imagePath)

	multiPartReq, err := newMultiPartRequest(imagePath, discordWebhook)
	if err != nil {
		fmt.Fprintln(os.Stderr, "Failed to create file multipart request:", err)
		os.Remove(imagePath)
		os.Exit(1)
	}

	if err := sendRequest(multiPartReq); err != nil {
		fmt.Fprintln(os.Stderr, "Failed to send capture to Discord:", err)
		os.Remove(imagePath)
		os.Exit(1)
	}

	fmt.Println("Capture posted to Discord channel.")
}

func captureImage() (string, error) {
	output, err := ioutil.TempFile("", "capture*.jpg")
	if err != nil {
		return "", err
	}
	output.Close()

	cmd := exec.Command("libcamera-still", "--width", "1024", "--height", "768", "-o", output.Name())
	// cmd := exec.Command("fswebcam", "-r", "1024x768", "--no-banner", output.Name())
	if err := cmd.Run(); err != nil {
		return "", fmt.Errorf("failed to capture image: %s", err)
	}

	return output.Name(), nil
}

func newMultiPartRequest(path, discordWebhook string) (*http.Request, error) {
	f, err := os.Open(path)
	if err != nil {
		return nil, err
	}
	defer f.Close()

	body := bytes.Buffer{}
	bodywriter := multipart.NewWriter(&body)

	part, err := bodywriter.CreateFormFile("file", filepath.Base(f.Name()))
	if err != nil {
		return nil, err
	}

	io.Copy(part, f)
	bodywriter.Close()

	request, err := http.NewRequest(http.MethodPost, discordWebhook, &body)
	if err != nil {
		return nil, err
	}
	request.Header.Add("Content-Type", bodywriter.FormDataContentType())

	return request, nil
}

func sendRequest(request *http.Request) error {
	client := http.Client{
		Timeout: 10 * time.Second,
	}
	response, err := client.Do(request)
	if err != nil {
		return err
	}
	defer response.Body.Close()

	if response.StatusCode != http.StatusOK {
		return fmt.Errorf("invalid response from discord channel: %s", response.Status)
	}

	return nil
}
