package main

import (
	"flag"
	"fmt"
	"log"
	"math"
	"net/http"
	"os"
	"time"

	"github.com/prometheus/client_golang/prometheus"
	"github.com/prometheus/client_golang/prometheus/promauto"
	"github.com/prometheus/client_golang/prometheus/promhttp"

	owm "github.com/briandowns/openweathermap"
	hue "github.com/collinux/gohue"
)

const rootPageHTML = `<html>
<head><title>External Weather Temperature Exporter</title></head>
<body>
  <h1>External Weather Temperature Exporter</h1>
  <p><a href='/metrics'>metrics</a></p>
</body>
</html>`

func lightweather(cfg *config, chRefresh <-chan struct{}) {
	externalWeatherTemp := promauto.NewGauge(prometheus.GaugeOpts{
		Name: "external_weather_temperature",
	})

	run := func() {
		log.Println("INFO: Getting current temperature")
		currentTemp, err := getCurrentTemperature(cfg)
		if err != nil {
			log.Println("ERROR:", err)
		}

		externalWeatherTemp.Set(float64(currentTemp))

		log.Println("INFO: Setting light")
		if err := setLight(cfg, currentTemp); err != nil {
			log.Println("ERROR:", err)
		}
	}

	for {
		select {
		case <-chRefresh:
			run()
		case <-time.Tick(30 * time.Minute):
			run()
		}
	}
}

func newMux(cfg *config) http.Handler {
	mux := http.NewServeMux()

	mux.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
		log.Println("INFO: Received request root")
		w.Write([]byte(rootPageHTML))
	})

	chRefresh := make(chan struct{}, 2)

	go lightweather(cfg, chRefresh)

	chRefresh <- struct{}{}

	mux.HandleFunc("/refresh", func(w http.ResponseWriter, r *http.Request) {
		log.Println("INFO: Received refresh request")

		if r.Method != http.MethodPost {
			msg := fmt.Sprintf("Method %s not allowed", r.Method)
			log.Println("ERROR:", msg)
			http.Error(w, msg, http.StatusMethodNotAllowed)
			return
		}

		chRefresh <- struct{}{}
	})

	mux.Handle("/metrics", promhttp.Handler())

	return mux
}

func main() {
	c := flag.String("c", "config.yml", "Config file")
	flag.Parse()

	cfg, err := newConfig(*c)
	if err != nil {
		log.Println("ERROR:", err)
		os.Exit(1)
	}

	s := &http.Server{
		Addr:         ":3040",
		Handler:      newMux(cfg),
		ReadTimeout:  10 * time.Second,
		WriteTimeout: 10 * time.Second,
	}

	if err := s.ListenAndServe(); err != nil {
		log.Println("ERROR:", err)
		os.Exit(1)
	}
}

func getCurrentTemperature(cfg *config) (int, error) {
	w, err := owm.NewCurrent(cfg.Unit, cfg.Lang, cfg.OWMAPIKey)
	if err != nil {
		return 0, err
	}

	err = w.CurrentByName(cfg.Location)
	return int(math.Round(w.Main.Temp)), err
}

func setLight(cfg *config, currentTemp int) error {
	bridge, err := hue.NewBridge(cfg.HueIPAddress)
	if err != nil {
		return err
	}

	// hue-id, _ := bridge.CreateUser("create-new-user")
	if err := bridge.Login(cfg.HueID); err != nil {
		return err
	}

	weatherLight, err := bridge.GetLightByName(cfg.LightName)
	if err != nil {
		return err
	}

	if err := weatherLight.SetColor(
		pickColor(cfg, currentTemp)); err != nil {
		return err
	}

	return weatherLight.On()
}
